aboutsummaryrefslogtreecommitdiff
path: root/src
diff options
context:
space:
mode:
Diffstat (limited to 'src')
-rw-r--r--src/nvim/move.c29
1 files changed, 19 insertions, 10 deletions
diff --git a/src/nvim/move.c b/src/nvim/move.c
index 144384c294..e7f5959dbb 100644
--- a/src/nvim/move.c
+++ b/src/nvim/move.c
@@ -1210,20 +1210,24 @@ bool scrolldown(long line_count, int byfold)
}
if (curwin->w_cursor.lnum == curwin->w_topline && do_sms) {
+ long so = curwin->w_p_so >= 0 ? curwin->w_p_so : p_so;
+ long scrolloff_cols = so == 0 ? 0 : width1 + (so - 1) * width2;
+
// make sure the cursor is in the visible text
validate_virtcol();
- int col = curwin->w_virtcol - curwin->w_skipcol;
+ long col = curwin->w_virtcol - curwin->w_skipcol + scrolloff_cols;
int row = 0;
if (col >= width1) {
col -= width1;
- ++row;
+ row++;
}
if (col > width2) {
- row += col / width2;
+ row += (int)col / width2;
col = col % width2;
}
if (row >= curwin->w_height) {
- coladvance(curwin->w_virtcol - (row - curwin->w_height + 1) * width2);
+ curwin->w_curswant = curwin->w_virtcol - (row - curwin->w_height + 1) * width2;
+ coladvance(curwin->w_curswant);
}
}
return moved;
@@ -1329,20 +1333,25 @@ bool scrollup(long line_count, int byfold)
}
if (curwin->w_cursor.lnum == curwin->w_topline && do_sms && curwin->w_skipcol > 0) {
- // make sure the cursor is in a visible part of the line
+ int width1 = curwin->w_width - curwin_col_off();
+ int width2 = width1 + curwin_col_off2();
+ long so = curwin->w_p_so >= 0 ? curwin->w_p_so : p_so;
+ long scrolloff_cols = so == 0 ? 0 : width1 + (so - 1) * width2;
+
+ // Make sure the cursor is in a visible part of the line, taking
+ // 'scrolloff' into account, but using screen lines.
validate_virtcol();
- if (curwin->w_virtcol < curwin->w_skipcol + 3) {
- int width1 = curwin->w_width - curwin_col_off();
- int width2 = width1 + curwin_col_off2();
+ if (curwin->w_virtcol < curwin->w_skipcol + 3 + scrolloff_cols) {
colnr_T col = curwin->w_virtcol;
if (col < width1) {
col += width1;
}
- while (col < curwin->w_skipcol + 3) {
+ while (col < curwin->w_skipcol + 3 + scrolloff_cols) {
col += width2;
}
- coladvance(col);
+ curwin->w_curswant = col;
+ coladvance(curwin->w_curswant);
}
}